package com.holmos.webtest.junitextentions.annotations;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * 加上此注解的测试方法，在测试用例运行失败的时候会重新进行测试
 * 
 * 对一些不稳定的应用和系统的测试，比如web ui层的测试，那么有很多用例就会失败，这些失败跟用例本身的写法有一定关系，但是从一定程度上是所测系统的硬伤，那么对于这种用例就需要在其失败的时候，进行重新测试，就等于是给了多次机会，至于多少次，使用者可以自己设定。
 * @Retry 注解可以使类和方法两种级别，注解里面包含一个属性int count,count表示可以重新测试的最大次数。Cancel 如果为true，则当前方法或者类不做重复性测试，
 * 当类级别的注解和方法级别的都有@Retry注解的时候，以方法级别的为准。
 * @author: 吴银龙(sunshine307087558@gmail.com)
 * @version: 2013-3-21 下午5:12:38 
 */
@Retention(RetentionPolicy.RUNTIME)

public @interface Retry {
	/**测试用例失败之后重复的次数，不算用例本身的第一次运行，默认为 0*/
	int count() default 0;
	/**是否要取消失败重新运行，默认为false，为true的话测试方法将不会在失败之后重复运行*/
	boolean cancel() default false;
}
